package betterwithmods.common.blocks.tile.gen;

import betterwithmods.api.block.IMechanicalBlock;
import betterwithmods.common.BWMBlocks;
import betterwithmods.common.blocks.mechanical.BlockAxle;
import betterwithmods.common.blocks.mechanical.BlockWindmill;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.DimensionType;
import net.minecraftforge.fml.common.Loader;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public class TileEntityWindmillHorizontal extends TileEntityMillGenerator implements IColor {
    public int[] bladeMeta = {0, 0, 0, 0};

    public TileEntityWindmillHorizontal() {
        super();
        this.radius = 7;
    }

    @Override
    public int getMinimumInput(EnumFacing facing) {
        return 0;
    }

    public int getBladeColor(int blade) {
        return bladeMeta[blade];
    }

    @Override
    public int getColorFromBlade(int blade) {
        return bladeMeta[blade];
    }

    @Override
    public boolean dyeBlade(int dyeColor) {
        boolean dyed = false;
        if (bladeMeta[dyeIndex] != dyeColor) {
            bladeMeta[dyeIndex] = dyeColor;
            dyed = true;
            IBlockState state = func_145831_w().func_180495_p(this.field_174879_c);
            this.func_145831_w().func_184138_a(this.field_174879_c, state, state, 3);
            this.func_70296_d();
        }
        dyeIndex++;
        if (dyeIndex > 3)
            dyeIndex = 0;
        return dyed;
    }

    @Override
    public void func_145839_a(NBTTagCompound tag) {
        super.func_145839_a(tag);
        for (int i = 0; i < 4; i++) {
            if (tag.func_74764_b("Color_" + i))
                bladeMeta[i] = tag.func_74762_e("Color_" + i);
        }
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound tag) {
        NBTTagCompound t = super.func_189515_b(tag);
        for (int i = 0; i < 4; i++) {
            t.func_74768_a("Color_" + i, bladeMeta[i]);
        }
        t.func_74774_a("DyeIndex", this.dyeIndex);
        return t;
    }

    @Override
    public void updateSpeed() {
        byte speed = 0;
        if (this.isValid() && !isGalacticraftDimension() && isNotOtherDimension()) {
            if (this.func_145831_w().field_73011_w.func_186058_p() == DimensionType.OVERWORLD && (this.func_145831_w().func_72896_J() || this.func_145831_w().func_72911_I()))
                speed = 2;
            else
                speed = 1;
        }
        if (speed != this.runningState || (speed == 0 && this.func_145831_w().func_180495_p(field_174879_c).func_177229_b(BlockWindmill.ISACTIVE))) {
            this.setRunningState(speed);
            this.func_145831_w().func_175656_a(field_174879_c, this.func_145831_w().func_180495_p(field_174879_c).func_177226_a(BlockWindmill.ISACTIVE, speed > 0));
            func_145831_w().func_180497_b(field_174879_c, this.func_145838_q(), this.func_145838_q().func_149738_a(func_145831_w()), 5);//this.getWorld().markBlockForUpdate(pos);
        }
    }

    public boolean isNotOtherDimension() {
        return this.func_145831_w().field_73011_w.func_186058_p() != DimensionType.NETHER && this.func_145831_w().field_73011_w.func_186058_p() != DimensionType.THE_END;
    }

    public boolean isGalacticraftDimension() {
        boolean isDimension = false;
        if (Loader.isModLoaded("GalacticraftCore")) {
            isDimension = false;//GalacticraftCompat.isGalacticraftDimension(this.getWorld());
        }
        return isDimension;
    }

    @Override
    public void overpower() {
        if (this.func_145838_q() instanceof BlockWindmill) {
            EnumFacing.Axis axis = func_145831_w().func_180495_p(field_174879_c).func_177229_b(BlockWindmill.AXIS);
            for (EnumFacing dir : EnumFacing.field_82609_l) {
                if (dir.func_176740_k() == axis) {
                    BlockPos offset = field_174879_c.func_177972_a(dir);
                    Block axle = this.func_145831_w().func_180495_p(offset).func_177230_c();
                    if (axle instanceof BlockAxle)
                        ((BlockAxle) axle).overpower(this.func_145831_w(), offset);
                    else if (axle instanceof IMechanicalBlock && ((IMechanicalBlock) axle).canInputPowerToSide(func_145831_w(), offset, dir.func_176734_d()))
                        ((IMechanicalBlock) axle).overpower(this.func_145831_w(), offset);
                }
            }
        }
    }

    @Override
    public boolean isValid() {
        boolean valid = true;
        if (func_145831_w().func_180495_p(field_174879_c).func_177230_c() != null && func_145831_w().func_180495_p(field_174879_c).func_177230_c() == BWMBlocks.WINDMILL_BLOCK) {
            EnumFacing.Axis axis = func_145831_w().func_180495_p(field_174879_c).func_177229_b(BlockWindmill.AXIS);
            for (int vert = -6; vert <= 6; vert++) {
                for (int i = -6; i <= 6; i++) {
                    int xP = (axis == EnumFacing.Axis.Z ? i : 0);
                    int zP = (axis == EnumFacing.Axis.X ? i : 0);
                    BlockPos offset = field_174879_c.func_177982_a(xP, vert, zP);
                    if (xP == 0 && vert == 0 && zP == 0)
                        continue;
                    else
                        valid = func_145831_w().func_175623_d(offset);
                    if (!valid)
                        break;
                }
                if (!valid)
                    break;
            }
        }
        return valid && this.func_145831_w().func_175710_j(field_174879_c);
    }

    //Extend the bounding box if the TESR is bigger than the occupying block.
    @Override
    @SideOnly(Side.CLIENT)
    public AxisAlignedBB getRenderBoundingBox() {
        int x = field_174879_c.func_177958_n();
        int y = field_174879_c.func_177956_o();
        int z = field_174879_c.func_177952_p();
        if (func_145831_w().func_180495_p(field_174879_c).func_177230_c() != null && func_145831_w().func_180495_p(field_174879_c).func_177230_c() instanceof BlockWindmill) {
            EnumFacing.Axis axis = func_145831_w().func_180495_p(field_174879_c).func_177229_b(BlockWindmill.AXIS);
            int xP = axis == EnumFacing.Axis.Z ? radius : 0;
            int yP = radius;
            int zP = axis == EnumFacing.Axis.X ? radius : 0;
            return new AxisAlignedBB(x - xP - 1, y - yP - 1, z - zP - 1, x + xP + 1, y + yP + 1, z + zP + 1);
        } else
            return super.getRenderBoundingBox();
    }
}
